﻿Module Module1

    Private card As New K8055()
    Private commandSet As New Collection()


#Region "Command Set"
    ' The Command Set section of the code contains the actual commands you can enter
    ' in the DOS/Telnet window.
    '
    ' I have created a very small instruction set with only 4 commands:
    ' CLS, QUIT, SETCHANNEL and CLEARCHANNEL.
    '
    '
    ' You can expand the instruction set by adding some extra commands in the loadCommands()
    ' sub. There are 2 possible syntaxes:
    '  addCommand(<name>,<delegate>,<number_of_parameters>)
    '  addCommand(<name>,<delegate>,<number_of_parameters>,<min_value_of_parameter><max_value_of_parameter>)
    '
    '   name                    : a unique name for your command.
    '   delegate                : the name of the subroutine that is called when you execute the command.
    '   number_of_parameters    : the number of parameters your function needs. It can be either 0 or 1.
    '   min_value_of_parameter  : the minimal value of your parameter
    '   max_value_of_parameter  : the maximal value of your parameter
    '
    Private Sub loadCommands()
        commandSet = New Collection()
        addCommand("CLS", AddressOf clearscreen, 0)
        addCommand("QUIT", AddressOf quit, 0)
        addCommand("SETCHANNEL", AddressOf setchannel, 1, 1, 8)
        addCommand("CLEARCHANNEL", AddressOf clearchannel, 1, 1, 8)
    End Sub


    ' You need to add a Sub routine for each command you add. The signature of the routine has to match
    ' the delegate signature.
    '
    ' Public Delegate Sub doAction(ByVal waarde As Integer)
    '
    Private Sub clearscreen(ByVal waarde As Integer)
        ' CLS function
        Console.Clear()
    End Sub

    Private Sub setchannel(ByVal waarde As Integer)
        ' SETCHANNEL function
        card.setDigitalOutput(waarde)
    End Sub

    Private Sub clearchannel(ByVal waarde As Integer)
        ' CLEARCHANNEL function
        card.clearDigitalOutput(waarde)
    End Sub

    Private Sub quit(ByVal waarde As Integer)
        ' QUIT function
        card.disconnect()
        End
    End Sub
#End Region





    ' These 2 subroutines will add a new Command() object to the collection with commands.
    Private Sub addCommand(ByVal name As String, ByVal subToCall As Command.doAction, ByVal expectedParameters As Integer)
        commandSet.Add(New Command(name, subToCall, expectedParameters), name)
    End Sub
    Private Sub addCommand(ByVal name As String, ByVal subToCall As Command.doAction, ByVal expectedParameters As Integer, ByVal minValue As Integer, ByVal maxValue As Integer)
        commandSet.Add(New Command(name, subToCall, expectedParameters, minValue, maxValue), name)
    End Sub



    ' Main sub of the program...
    Sub Main()
        ' Load all the existing commands
        loadCommands()

        ' Welcome to the K8055 Telnet Program
        Console.WriteLine("Welcome to the K8055 Telnet program")
        Console.WriteLine("This program enables you to control K8055 over DOS/Telnet")
        Console.WriteLine("")
        Console.WriteLine("")

        card.connect(SK5:=True, SK6:=True)

        If Not card.isConnected() Then
            Console.WriteLine("Error: the K8055 is not found!")
        Else
            Console.WriteLine("Connection Established!")
            Console.WriteLine("You can now enter one of these commands:")

            ' Show a listing of all available commands
            For Each cmd As Command In commandSet
                If cmd.minvalue <> -1 Or cmd.maxValue <> -1 Then
                    Console.WriteLine(cmd.name & " " & "[" & cmd.minvalue & "-" & cmd.maxValue & "]")
                Else
                    Console.WriteLine(cmd.name)
                End If


            Next

            ' This is the part of the program that will be cycled until you enter the
            ' Quit command.
            Do
                Dim newCmd As String()
                newCmd = getInput("> ")

                If UBound(newCmd) > -1 Then
                    ' One or more parameters are given
                    If commandSet.Contains(newCmd(0)) Then
                        Dim calledCommand As Command = commandSet.Item(newCmd(0))
                        Dim res As String
                        res = calledCommand.execute(newCmd)
                        If Not (res Is Nothing) Then
                            Console.WriteLine("Error: " & res)
                        End If
                    Else
                        Console.WriteLine("Unknown Command")
                    End If
                End If
            Loop

        End If

        ' Line added for debugging purposes. You can remove it if you want.
        Console.ReadLine()
    End Sub



    ' GetInput will read the entered data, convert to uppercase, and split the String into an Array.
    Private Function getInput(ByVal text As String) As String()
        Console.Write(text)
        Dim commandText As String
        commandText = Console.ReadLine()
        commandText = commandText.ToUpper()
        commandText = commandText.Trim()
        commandText = commandText.Replace(vbTab, " ")

        ' Remove double spaces
        Do
            commandText = commandText.Replace("  ", " ")
        Loop While commandText.Contains("  ")


        getInput = commandText.Split(" ")
    End Function
End Module